package RevisionDiff::CMS;

use strict;
use warnings;

use MT::Util qw( format_ts relative_date );
use Text::Diff;

sub diff_revision {
    my $app = shift;

    my $q     = $app->param;
    my $type  = $q->param('_type');
    my $id    = $q->param('id');
    my $blog_id = $q->param('blog_id');
    my $rev   = $q->param('r');

    $app->call_return if $rev == 1;

    return $app->errtrans("Invalid request.")
        unless $type;

    my $class = $app->model($type)
        or return $app->errtrans( "Invalid type [_1]", $type );

    return $app->error( $app->translate( "No ID" ) ) if !$id;

    my $e = $class->load($id);
    my $rev_new = $e->load_revision( { rev_number => $rev } );
    my $obj_new = $rev_new->[0];

    my $old = $rev - 1;

    my $rev_old = $e->load_revision( { rev_number => $old } );
    my $obj_old = $rev_old->[0];

    my $text_new = $obj_new->text;
    my $text_old = $obj_old->text;

    my $diff = diff \$text_old, \$text_new;
    $diff =~ s/</&lt;/g;
    $diff =~ s/>/&gt;/g;

   my $static_path = $app->static_path;
   my $cancel = MT->translate('Cancel');

return <<EOF;
<!DOCTYPE html>
<html lang="ja">
<head>
  <meta http-equiv="X-UA-Compatible" content="IE=Edge" />
  <meta charset="UTF-8" />
  <link rel="stylesheet" href="${static_path}css/main.css" type="text/css" />
</head>
<body id="" class="dialog dialog-screen list-revision-dialog">
  <div id="container">
    <div id="content">
      <div id="content-header">
        <h1 id="page-title" class="title page">diff revision$old and revision$rev</h1>
      </div>
      <div id="content-body">
<pre>
$diff;
</pre>
<div class="actions-bar actions-bar-bottom line">
<button
    type="submit"
    accesskey="x"
    class="cancel action button mt-close-dialog"
    onclick="javascript:history.back();"
    title="$cancel"
    >$cancel</button>
</div>
      </div>
    </div>
  </div>
</body>
</html>
EOF
}

sub revision_table {
    my ($cb, $app, $terms, $args, $param, $hasher) = @_;

    my $blog_id  = $app->param('blog_id');
    my $id = $app->param('id');

    my $q     = $app->param;
    my $type  = $q->param('_type');
    my $class = $app->model($type);
    my $param = $args->{param};

    my $obj   = $class->load($id);
    my $blog  = $app->blog;
    my $lang  = $app->user ? $app->user->preferred_language : undef;

    my $js = $param->{rev_js};
    unless ($js) {
        $js
            = "location.href='"
            . $app->uri
            . '?__mode=view&amp;_type='
            . $type;
        if ( my $id = $obj->id ) {
            $js .= '&amp;id=' . $id;
        }
        if ( defined $blog ) {
            $js .= '&amp;blog_id=' . $blog->id;
        }
    }
    my %users;

    my $js2 = "location.href='"
            . $app->uri
            . '?__mode=diff_revision&amp;_type='
            . $type;
        if ( my $id = $obj->id ) {
            $js2 .= '&amp;id=' . $id;
        }
        if ( defined $blog ) {
            $js2 .= '&amp;blog_id=' . $blog->id;
        }


    $$hasher = sub {
        my ( $rev, $row ) = @_;
        if ( my $ts = $rev->created_on ) {
            $row->{created_on_formatted}
                = format_ts( MT::App::CMS::LISTING_DATE_FORMAT(),
                $ts, $blog, $lang );
            $row->{created_on_time_formatted}
                = format_ts( MT::App::CMS::LISTING_TIMESTAMP_FORMAT(),
                $ts, $blog, $lang );
            $row->{created_on_relative} = relative_date( $ts, time, $blog );
        }
        if ( $row->{created_by} ) {
            my $created_user = $users{ $row->{created_by} }
                ||= MT::Author->load( $row->{created_by} );
            if ($created_user) {
                $row->{created_by} = $created_user->nickname;
            }
            else {
                $row->{created_by} = $app->translate('(user deleted)');
            }
        }
        my $revision    = $obj->object_from_revision($rev);
        my $column_defs = $obj->column_defs;

        #my @changed = map {
        #    my $label = $column_defs->{$_}->{label};
        #    $label ||= $_;
        #    $app->translate( $label );
        #} @{ $revision->[1] };
        #$row->{changed_columns} = \@changed;
        if ( ( 'entry' eq $type ) || ( 'page' eq $type ) ) {
            $row->{rev_status} = $revision->[0]->status;
        }
        $row->{rev_js}     = $js . '&amp;r=' . $row->{rev_number} . "'";
        $row->{rev_js2}    = $js2 . '&amp;r=' . $row->{rev_number} . "'";
MT->instance->log($row->{rev_js2});
        $row->{is_current} = $param->{revision} == $row->{rev_number};
    };


}

sub list_revision {
    my ($cb, $app, $terms, $args, $param, $hasher) = @_;

    my $q = $app->param;
    my $r = $q->param('r');

    my $id = $terms->{entry_id};
    my $obj = MT::Entry->load($id);
    my $type = 'entry';
    my $blog = $app->blog;
    my $lang  = $app->user ? $app->user->preferred_language : undef;
    my %users;
    my $js = $param->{rev_js};
    unless ($js) {
        $js
            = "location.href='"
            . $app->uri
            . '?__mode=view&amp;_type='
            . $app->param('_type');
        if ( my $id = $obj->id ) {
            $js .= '&amp;id=' . $id;
        }
        if ( defined $blog ) {
            $js .= '&amp;blog_id=' . $blog->id;
        }
    }
    $$hasher = sub {
        my ( $rev, $row ) = @_;

        if ( my $ts = $rev->created_on ) {
            $row->{created_on_formatted}
                = format_ts( MT::App::CMS::LISTING_DATE_FORMAT(),
                $ts, $blog, $lang );
            $row->{created_on_time_formatted}
                = format_ts( MT::App::CMS::LISTING_TIMESTAMP_FORMAT(),
                $ts, $blog, $lang );
            $row->{created_on_relative} = relative_date( $ts, time, $blog );
        }
        if ( $row->{created_by} ) {
            my $created_user = $users{ $row->{created_by} }
                ||= MT::Author->load( $row->{created_by} );
            if ($created_user) {
                $row->{created_by} = $created_user->nickname;
            }
            else {
                $row->{created_by} = $app->translate('(user deleted)');
            }
        }
        my $revision    = $obj->object_from_revision($rev);
        my $column_defs = $obj->column_defs;

        if ( ( 'entry' eq $type ) || ( 'page' eq $type ) ) {
            $row->{rev_status} = $revision->[0]->status;
        }
        $row->{rev_js}  = $js . '&amp;r=' . $row->{rev_number} . "'";

        my ($terms, $args);
        $terms->{'entry_id'} = $revision->[0]->id;
        $args->{sort} = 'rev_number';
        $args->{direction} = 'descend';
        $args->{limit} = 1;
        my $rev_model = MT->instance->model($type.':revision');
        my @revs = $rev_model->load($terms, $args);
        my $revision = shift @revs || undef;

        if ($r) {
            $row->{is_current} = $r == $row->{rev_number};
        } else {
            $row->{is_current} = $revision->rev_number == $row->{rev_number};
        }
    };

}

sub add_column {
    my ($cb, $app, $template) = @_;

    my $old;
    my $new;

    $old = <<HTML;
      <th class="col head created-by"><span class="col-label"><__trans phrase="Saved By"></span></th>
HTML
    $old = quotemeta($old);

    $new = <<HTML;
      <th class="col head"><span class="col-label">diff</span></th>
      <th class="col head created-by"><span class="col-label"><__trans phrase="Saved By"></span></th>
HTML

#    $$template =~ s/$old/$new/;

    $old = <<HTML;
      <td class="col created-by"><span class="revision-by"><mt:var name="created_by" escape="html"></span></td>
HTML
    $old = quotemeta($old);

    $new = <<HTML;
      <td class="col created-by"><span class="revision-by"><a class="revision-number" href="javascript:void(0);" onclick="javascript:<mt:var name="rev_js2"\>;"><mt:var name="created_by" escape="html"></a></span></td>
HTML
    $$template =~ s/$old/$new/;
}

1;
